/************************************************************************/
/* metric.h								*/
/************************************************************************/
/*
 *  Copyright 2022 Todd Shadburn <tshadburn@opencomputingsolutions.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/



#ifndef _METRIC_H_
#define _METRIC_H_


#include <stdint.h>



#define METRIC_API	static __attribute__((always_inline)) inline


typedef enum {
	METRIC_TYPE_NONE	= 0,
	METRIC_TYPE_UINT32,
	METRIC_TYPE_UINT64,
	METRIC_TYPE_FLOAT,
	METRIC_TYPE_DOUBLE,
} metric_type_t;

typedef struct _metric_t metric_t;
struct _metric_t {
	char		*name;
	union {
		uint32_t	u32;
		uint64_t	u64;
		float		f;
		double		d;
	} u;
	uint8_t		type;
};


METRIC_API void metric_init(metric_t *m, char *name, metric_type_t type)
{
	m->name = name;
	m->type = type;
	m->u.u64 = 0L;
}


METRIC_API void metric_incr_u32(metric_t *m)
{
	m->u.u32++;
}


METRIC_API void metric_set_u32(metric_t *m, uint32_t v)
{
	m->u.u32 = v;
}


METRIC_API void metric_incr_u64(metric_t *m)
{
	m->u.u64++;
}


METRIC_API void metric_set_u64(metric_t *m, uint64_t v)
{
	m->u.u64 = v;
}


METRIC_API void metric_set_float(metric_t *m, float v)
{
	m->u.f = v;
}


METRIC_API void metric_set_double(metric_t *m, double v)
{
	m->u.d = v;
}


METRIC_API char *metric_get_name(metric_t *m)
{
	return m->name;
}


METRIC_API uint8_t metric_get_type(metric_t *m)
{
	return m->type;
}


METRIC_API uint32_t metric_get_u32(metric_t *m)
{
	return m->u.u32;
}


METRIC_API uint64_t metric_get_u64(metric_t *m)
{
	return m->u.u64;
}


METRIC_API float metric_get_float(metric_t *m)
{
	return m->u.f;
}


METRIC_API double metric_get_double(metric_t *m)
{
	return m->u.d;
}




#endif


